PORTA		EQU	2000H
PORTB		EQU	2001H
PORTC		EQU	2002H
CWR		EQU	2003H
CWO		EQU	  80H
CWI		EQU	  90H	

;-----------
;LCD Command
;-----------
DisplayClear	        EQU	01H
CursorHome	        EQU	02H
ShiftCursorRight        EQU	14H
ShiftCursorLeft		EQU	10H
ShiftDisplayRight	EQU	1CH
ShiftDisplayLeft	EQU	18H
DShiftCursorInc		EQU	07H
DShiftCursorDec		EQU	05H

;		------------------
;		Long Delay Routine
;		Input : None
;		Output : None
;		------------------
LDelay:		PUSH	06H
		PUSH	07H
		MOV	R6,#0FFH
LDel:		MOV	R7,#0FFH
		DJNZ	R7,$
		DJNZ	R6,LDel
		POP	07H
		POP	06H
		RET
;		---------------------------
;		8255 Initialization Routine
;		Input : A = Control word
;		Output : None
;		---------------------------
InitPPI:	MOV	DPTR,#CWR
		MOVX	@DPTR,A
		RET		
;		-------------------------------------
;		Write to Instruction Register Routine
;		Input : A = Data to write
;		Output : None
;		-------------------------------------
WRIR:		MOV	DPTR,#PORTA
		MOVX	@DPTR,A
		MOV	DPTR,#PORTB
		MOV	A,#00000100B		;E=1,R/W=0,RS=0
		MOVX	@DPTR,A
		MOV	A,#00000000B		;E=0,R/W=0,RS=0
		MOVX	@DPTR,A
		RET
;		------------------------------
;		Write to Data Register Routine
;		Input : A = Data to write
;		Output : None
;		------------------------------
WRDR:		MOV	DPTR,#PORTA
		MOVX	@DPTR,A
		MOV	DPTR,#PORTB
		MOV	A,#00000101B		;E=1,R/W=0,RS=1
		MOVX	@DPTR,A
		MOV	A,#00000001B		;E=0,R/W=0,RS=1
		MOVX	@DPTR,A
		MOV	A,#00000000B		;E=0,R/W=0,RS=0
		MOVX	@DPTR,A
		RET
;		-------------------------------
;		Read from Data Register Routine
;		Input : None
;		Output : A = Data read
;		------------------------------
RDDR:		MOV	A,#CWI
		LCALL	InitPPI
		MOV	DPTR,#PORTB
		MOV	A,#00000111B		;E=1,R/W=1,RS=1
		MOVX	@DPTR,A
		MOV	DPTR,#PORTA
		MOVX	A,@DPTR
		PUSH	ACC
		MOV	DPTR,#PORTB
		MOV	A,#00000011B		;E=0,R/W=1,RS=1
		MOVX	@DPTR,A
		MOV	A,#00000000B		;E=0,R/W=0,RS=0
		MOVX	@DPTR,A
		MOV	A,#CWO
		LCALL	InitPPI
		POP	ACC
		RET
;		------------------------
;		Check Busy Flag Routine
;		Input : None
;		Output : None
;		------------------------
CBF:		PUSH	ACC
		PUSH	B
		PUSH	DPH
		PUSH	DPL
		MOV	A,#CWI
		LCALL	InitPPI
		MOV	DPTR,#PORTB
W4Ready:	MOV	A,#00000110B		;E=1,R/W=1,RS=0
		MOVX	@DPTR,A
		MOV	DPTR,#PORTA
		MOVX	A,@DPTR
		PUSH	ACC
		MOV	DPTR,#PORTB
		MOV	A,#00000010B		;E=0,R/W=1,RS=0
		MOVX	@DPTR,A
		MOV	A,#00000000B		;E=0,R/W=0,RS=0
		MOVX	@DPTR,A
		POP	ACC
		JB	ACC.7,W4Ready
		MOV	A,#CWO
		LCALL	InitPPI
		POP	DPL
		POP	DPH
		POP	B
		POP	ACC
		RET
;		---------------------------------------
;		Initialize LCD for 4 bit data interface
;		Input : None
;		Output : None
;		----------------------------------------
InitLCD:	PUSH	ACC
		PUSH	B
		PUSH	DPH
		PUSH	DPL
		LCALL	LDelay
		MOV	A,#CWO
		LCALL	InitPPI
NxtInit:	MOV	A,#00110000B		;DB0-7=0011XXXXB
		LCALL	WRIR
		LCALL	LDelay
		MOV	A,#00110000B		;DB0-7=0011XXXXB
		LCALL	WRIR
		LCALL	LDelay
		MOV	A,#00110000B		;DB0-7=0011XXXXB
		LCALL	WRIR
		LCALL	CBF
		MOV	A,#00111000B		;8bit interface
		LCALL	WRIR
		LCALL	CBF		
		MOV	A,#00001000B		;Display off
		LCALL	WRIR
		LCALL	CBF
		MOV	A,#00000001B		;Display clear
		LCALL	WRIR
		LCALL	CBF
		MOV	A,#00000110B		;Entry mode set
		LCALL	WRIR
		LCALL	CBF
		MOV	A,#00001100B		;Display on
		LCALL	WRIR
		LCALL	CBF
		POP	DPL
		POP	DPH
		POP	B
		POP	ACC
		RET
;		----------------------
;		Write Command to LCD
;		Input : ACC = Command
;		Ouput : None
;		----------------------
CommandLCD:	PUSH	ACC
		PUSH	DPH
		PUSH	DPL
		LCALL	WRIR
		LCALL	CBF
		POP	DPL
		POP	DPH
		POP	ACC
		RET
;		-------------------
;		Write Data to LCD
;		Input : ACC = Data
;		Ouput : None
;		-------------------
WriteLCD:	PUSH	ACC
		PUSH	DPH
		PUSH	DPL
		LCALL	WRDR
		LCALL	CBF
		POP	DPL
		POP	DPH
		POP	ACC
		RET
;		-------------------
;		Read Data from LCD
;		Input : None
;		Ouput : ACC = Data
;		-------------------
ReadLCD:	PUSH	DPH
		PUSH	DPL
		LCALL	RDDR
		LCALL	CBF
		POP	DPL
		POP	DPH		
		RET
;		----------------------
;		Read LCD Address
;		Input : None
;		Ouput : ACC = Address
;		----------------------
ReadAddrLCD:	PUSH	DPH
		PUSH	DPL
		MOV	A,#CWI
		LCALL	InitPPI
		MOV	DPTR,#PORTB
LCDBusy:	MOV	A,#00000110B		;E=1,R/W=1,RS=0
		MOVX	@DPTR,A
		MOV	DPTR,#PORTA
		MOVX	A,@DPTR
		PUSH	ACC
		MOV	DPTR,#PORTB
		MOV	A,#00000010B		;E=0,R/W=1,RS=0
		MOVX	@DPTR,A
		MOV	A,#00000000B		;E=0,R/W=0,RS=0
		MOVX	@DPTR,A
		POP	ACC
		JB	ACC.7,LCDBusy
		PUSH	ACC
		MOV	A,#CWO
		LCALL	InitPPI
		POP	ACC
		POP	DPL
		POP	DPH
		RET
;		---------------------------------
;		Set Operation to DDRAM
;		Input : ACC = DDRAM Start Address
;		Output : None
;		---------------------------------
SetDDRAM:	PUSH	ACC
		PUSH	DPH
		PUSH	DPL
		SETB	ACC.7
		LCALL	WRIR
		LCALL	CBF
		POP	DPL
		POP	DPH
		POP	ACC
		RET
;		---------------------------------
;		Set Operation to CGRAM
;		Input : ACC = CGRAM Start Address
;		Output : None
;		---------------------------------
SetCGRAM:	PUSH	ACC
		PUSH	DPH
		PUSH	DPL
		CLR	ACC.7
		SETB	ACC.6
		LCALL	WRIR
		LCALL	CBF
		POP	DPL
		POP	DPH
		POP	ACC
		RET
